/*
###############################################################################
Include Part
###############################################################################
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>			// to use printf
#include <string.h>
#include <util/delay.h>
#include "serial.h" 			// serial related functions
#include "socket.h" 			
#include "types.h"
#include "w5200.h"
#include "lcd.h"
#include "spi.h"
#include "delay.h"
//#include "dhcp.h"
//#include "delay.h"

/*
###############################################################################
Define Part
###############################################################################
*/
#define	MAX_BUF_SIZE	2048		// Maximum receive buffer size

#define	ATMEGA128_0WAIT		0
#define	ATMEGA128_1WAIT		1
#define	ATMEGA128_2WAIT		2
#define	ATMEGA128_3WAIT		3
#define ATMEGA128_NUM_WAIT	ATMEGA128_3WAIT

//#define __DEF_TCPS_LOOPBACK__
//#define __DEF_TCPC_LOOPBACK__
//#define __DEF_UDP_LOOPBACK__
//#define __DEF_MACRAW_LOOPBACK__
//#define __DEF_WOL__
//#define __DEF_DHCP__
//#define __DEF_PPPOE_TEST__
#define __DEF_WINDOW_FULL__

/*
###############################################################################
Local  Variable Declaration Part
###############################################################################
*/
uint8 * data_buf = (uint8*) TX_BUF;	// Position of receive buffer
static u_char evb_lcd_text[LCD_MAX_ROW][LCD_MAX_COL+1];
void evb_set_lcd_text(u_char row, u_char* lcd);
#ifdef __DEF_DHCP__
extern u_char GET_SN_MASK[4];		/**< Subnet mask received from the DHCP server */
extern u_char GET_GW_IP[4];		/**< Gateway IP address received from the DHCP server */
extern u_char GET_DNS_IP[4];		/**< DNS server IP address received from the DHCP server */
extern u_char GET_SIP[4];		/**< Leased source IP address received from the DHCP server */
#endif
#ifdef __DEF_WINDOW_FULL__
uint8 test_ch;
uint8 test_data_len = 16;
uint8 test_data[] = "0123456789abcdef";
#endif

/*
###############################################################################
Function Prototype Declaration Part
###############################################################################
*/
void InitMCU(void); 	
void mcu_init(void);	
void loopback_tcps(SOCKET s, uint16 port, uint8 * data_buf, uint16 mode);
#ifdef __DEF_TCPC_LOOPBACK__
void loopback_tcpc(SOCKET s, uint8* addr, uint16 port, uint8 * data_buf, uint16 mode);
#endif
void loopback_udp(SOCKET s, uint16 port, uint8* data_buf, uint16 mode);
#ifdef __DEF_MACRAW_LOOPBACK__
void loopback_macraw(SOCKET s, uint16 port, uint8 * data_buf, uint16 mode);
#endif
void evb_soft_reset(void);

/*
###############################################################################
Function Implementation Part
###############################################################################
*/

int main(void)
{
#ifdef __DEF_DHCP__
	uint8 i;
#endif

	uint8 tmp[6] = {0,0,0,0,0,0};
	uint8 ip[4] = {192,168,11,129};                   // for setting SIP register
	uint8 gw[4] = {192,168,11,1};                     // for setting GAR register
	uint8 sn[4] = {255,255,255,0};                     // for setting SUBR register
	uint8 mac[6] = {0x00,0x08,0xDC,0x11,0x22,0x26};      // for setting SHAR register
	uint16 len= 0;
	uint16 recv_len = 0;
	uint16 port = 5001;
	uint16 mode =0;
	uint8 s =1;
	uint8 tmp_retry_cnt =0;
	uint16 send_data_len = 0;

	uint8 txsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};
	uint8 rxsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};

	uint16 addr;

	InitMCU();		

#if (__DEF_IINCHIP_BUS__ == __DEF_IINCHIP_INDIRECT_MODE__)
	*((volatile uint8*)(0x0000)) = 0x00;
	// H/W Reset W5200 Codes..
	// Wait for Reset Period Codes..
#endif

	iinchip_init();
	wait_1ms(1000);

	IINCHIP_WRITE((SHAR0 + 0),mac[0]);
	IINCHIP_WRITE((SHAR0 + 1),mac[1]);
	IINCHIP_WRITE((SHAR0 + 2),mac[2]);
	IINCHIP_WRITE((SHAR0 + 3),mac[3]);
	IINCHIP_WRITE((SHAR0 + 4),mac[4]);
	IINCHIP_WRITE((SHAR0 + 5),mac[5]);

	IINCHIP_WRITE((GAR0 + 0),gw[0]);
	IINCHIP_WRITE((GAR0 + 1),gw[1]);
	IINCHIP_WRITE((GAR0 + 2),gw[2]);
	IINCHIP_WRITE((GAR0 + 3),gw[3]);
	
	IINCHIP_WRITE((SUBR0 + 0),sn[0]);
	IINCHIP_WRITE((SUBR0 + 1),sn[1]);
	IINCHIP_WRITE((SUBR0 + 2),sn[2]);
	IINCHIP_WRITE((SUBR0 + 3),sn[3]);
	
	IINCHIP_WRITE((SIPR0 + 0),ip[0]);
	IINCHIP_WRITE((SIPR0 + 1),ip[1]);
	IINCHIP_WRITE((SIPR0 + 2),ip[2]);
	IINCHIP_WRITE((SIPR0 + 3),ip[3]);
  	sysinit(txsize,rxsize);

	printf("============================================\r\n");
	printf("   W5200 SPI mode  Net Config Information\r\n");
	printf("============================================\r\n");
	printf("MAC ADDRESS      : \r\n");

	tmp[0] = IINCHIP_READ(SHAR0+0);
	tmp[1] = IINCHIP_READ(SHAR0+1);
	tmp[2] = IINCHIP_READ(SHAR0+2);
	tmp[3] = IINCHIP_READ(SHAR0+3);
	tmp[4] = IINCHIP_READ(SHAR0+4);
	tmp[5] = IINCHIP_READ(SHAR0+5);

	printf("%02x.%02x.%02x.%02x.%02x.%02x\r\n",tmp[0],tmp[1],tmp[2],tmp[3],tmp[4],tmp[5]);	

	printf("SUBNET MASK      : \r\n"); 

	tmp[0] = IINCHIP_READ(SUBR0 + 0);
	tmp[1] = IINCHIP_READ(SUBR0 + 1);
	tmp[2] = IINCHIP_READ(SUBR0 + 2);
	tmp[3] = IINCHIP_READ(SUBR0 + 3);
	
	printf("%d.%d.%d.%d\r\n",tmp[0],tmp[1],tmp[2],tmp[3]);

	printf("G/W IP ADDRESS   : \r\n");
	
	tmp[0] = IINCHIP_READ(GAR0 + 0);
	tmp[1] = IINCHIP_READ(GAR0 + 1);
	tmp[2] = IINCHIP_READ(GAR0 + 2);
	tmp[3] = IINCHIP_READ(GAR0 + 3);
	
	printf("%d.%d.%d.%d\r\n",tmp[0],tmp[1],tmp[2],tmp[3]);

	printf("LOCAL IP ADDRESS : \r\n"); 
	
	tmp[0] = IINCHIP_READ(SIPR0 + 0);
	tmp[1] = IINCHIP_READ(SIPR0 + 1);
	tmp[2] = IINCHIP_READ(SIPR0 + 2);
	tmp[3] = IINCHIP_READ(SIPR0 + 3);

	
	printf("%d.%d.%d.%d\r\n",tmp[0],tmp[1],tmp[2],tmp[3]);
	printf("====================================\r\n");
	printf("AVR :TCP SERVER, PORT : %d\r\n", port);
	printf("====================================\r\n");

#ifdef __DEF_IINCHIP_INT__	
	IINCHIP_WRITE(IMR, 0x00);
	IINCHIP_WRITE(IMR2, 0xFF);
#endif

	while(1)
	{	
		switch(getSn_SR(s))
		{

			case SOCK_ESTABLISHED:						// If client is connected 
				if (getSn_IR(s) & Sn_IR_CON) 			// Confirm if received the data
				{
						printf("Connect OK : %d\r\n",s);
						IINCHIP_WRITE(Sn_IR(s),Sn_IR_CON);	// clear Sn_IR_CON
		       	}


				if( (len=getSn_RX_RSR(s)) > 0 )    // check the size of received data
				{
					//RECV
					recv_len = recv(s, data_buf, len);        // recv            				

					//SEND			
					init_windowfull_retry_cnt(s);			
					send_data_len = send(s, data_buf, recv_len, WINDOWFULL_FLAG_OFF);

					while(send_data_len != recv_len )
					{
						tmp_retry_cnt = incr_windowfull_retry_cnt(s);
						if( tmp_retry_cnt <= (WINDOWFULL_MAX_RETRY_NUM))
						{
							send_data_len += send(s, /* Socket Number */
												  test_data,  /* On WindowFull, this variable is not used */
												  test_data_len, /* On WindowFull, this variable is not used */
												  WINDOWFULL_FLAG_ON /*On WindowFull,  WINDOWFULL_FLAG_ON is '1' */
												  )	;  							  							
							wait_1ms(WINDOWFULL_WAIT_TIME); // Defaoult Wait TIME 1 Sec. defined on W5200.h
						}
						else
						{
							printf("WindowFull !!!\r\n"); 
							close(s);
							printf("Socket Close !!!\r\n"); 
							while(1);			  							
							}
					}
				}
				break;
			case SOCK_CLOSE_WAIT:                           		// If the client request to close connection and wait for closing successfully
				printf("CLOSE_WAIT : %d\r\n",s);					
				disconnect(s);
				break;
			case SOCK_CLOSED:                                               // If the socket connected to client already has been closed
				printf("CLOSED. CH : %d\r\n",s);	
				close(s);                        // close the SOCKET
				socket(s,Sn_MR_TCP,port,mode);   // open the SOCKET
				break;
			case SOCK_INIT:                     // The SOCKET opened with TCP mode
				listen(s);                       // listen to any connection request from "TCP CLIENT"
				printf("LOOPBACK_TCPSStarted. CH : %d\r\n",s);
				break;
			default:
				break;
		}
		
	}

}

/*
********************************************************************************
*              AVR128 Initialization Function
*
* Description: 
* Arguments  : None.
* Returns    : None.
* Note       : 
********************************************************************************
*/
void evb_set_lcd_text(u_char row, u_char* lcd)
{
	if(row < LCD_MAX_ROW && strlen((char*)lcd) <= LCD_MAX_COL)
	{
		memset(evb_lcd_text[row],' ',LCD_MAX_COL);
		evb_lcd_text[row][LCD_MAX_COL] = 0;
		memcpy(evb_lcd_text[row],lcd,strlen((char*)lcd));
		lcd_gotoxy(0,row);
		lcd_puts((char*)evb_lcd_text[row]);
	}
}

void InitMCU(void)
{ 
	mcu_init();
	lcd_init();
	evb_set_lcd_text(0,(u_char*)"W5200 Test");
	uart_init(0,7);
}

void mcu_init(void) 
{
	cli();
		
#ifndef __DEF_IINCHIP_INT__	
	EICRA=0x00;
	EICRB=0x00;
	EIMSK=0x00;
	EIFR=0x00;
#else
	EICRA = 0x00;			// External Interrupt Control Register A clear
	EICRB = 0x02;			// External Interrupt Control Register B clear // edge 
	EIMSK = 0x20;		// External Interrupt Mask Register : 0x10
	EIFR = 0xFF;			// External Interrupt Flag Register all clear
	DDRE &= ~0x20;		// Set PE Direction 
	PORTE |= 0x20;		// Set PE Default value
#endif


#if (ATMEGA128_NUM_WAIT == ATMEGA128_0WAIT)
	MCUCR = 0x80;
	XMCRA=0x40;
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_1WAIT)
	MCUCR = 0xc0;		// MCU control regiseter : enable external ram
	XMCRA=0x40;		// External Memory Control Register A : 
						// Low sector   : 0x1100 ~ 0x7FFF
						// Upper sector : 0x8000 ~ 0xFFFF
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_2WAIT)
	MCUCR = 0x80;
	XMCRA=0x42;
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_3WAIT)
	MCUCR = 0xc0;
	XMCRA=0x42;
#else
#error "unknown atmega128 number wait type"
#endif

	sei();				// enable interrupts
}





